// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

/******************************************************************

DO NOT MODIFY. AUTOGENERATED FILE.
This file is generated using the logic from <root>/src/scripts/genEventPipe.py

******************************************************************/


#include <common.h>
#include <gcenv.h>

#include <eventtrace_context.h>
#include <gcheaputilities.h>

#include <eventpipe/ep.h>
#include <eventpipe/ep-provider.h>
#include <eventpipe/ep-config.h>
#include <eventpipe/ep-event.h>
#include <eventpipe/ep-event-instance.h>
#include <eventpipe/ep-session.h>
#include <eventpipe/ep-session-provider.h>
#include <eventpipe/ep-metadata-generator.h>
#include <eventpipe/ep-event-payload.h>
#include <eventpipe/ep-buffer-manager.h>


#ifndef W
#define W(str) L##str
#endif
#ifndef SL
#define SL(str) str
#endif
#ifndef ERROR_SUCCESS
#define ERROR_SUCCESS 0L
#endif
#ifndef ERROR_WRITE_FAULT
#define ERROR_WRITE_FAULT 29L
#endif


bool ResizeBuffer(BYTE *&buffer, size_t& size, size_t currLen, size_t newSize, bool &fixedBuffer);
bool WriteToBuffer(const WCHAR* str, BYTE *&buffer, size_t& offset, size_t& size, bool &fixedBuffer);
bool WriteToBuffer(const BYTE *src, size_t len, BYTE *&buffer, size_t& offset, size_t& size, bool &fixedBuffer);

EventPipeProvider * create_provider(const WCHAR*, EventPipeCallback, void* pCallbackContext = nullptr);

template <typename T>
bool WriteToBuffer(const T &value, BYTE *&buffer, size_t& offset, size_t& size, bool &fixedBuffer)
{
    if (sizeof(T) + offset > size)
    {
        if (!ResizeBuffer(buffer, size, offset, size + sizeof(T), fixedBuffer))
            return false;
    }

    memcpy(buffer + offset, (char *)&value, sizeof(T));
    offset += sizeof(T);
    return true;
}


const WCHAR* DotNETRuntimePrivateName = W("Microsoft-Windows-DotNETRuntimePrivate");
EventPipeProvider *EventPipeProviderDotNETRuntimePrivate = nullptr;
EventPipeEvent *EventPipeEventGCDecision_V1 = nullptr;
EventPipeEvent *EventPipeEventGCSettings_V1 = nullptr;
EventPipeEvent *EventPipeEventGCOptimized_V1 = nullptr;
EventPipeEvent *EventPipeEventGCGlobalHeapHistory = nullptr;
EventPipeEvent *EventPipeEventPrvGCMarkStackRoots_V1 = nullptr;
EventPipeEvent *EventPipeEventPrvGCMarkFinalizeQueueRoots_V1 = nullptr;
EventPipeEvent *EventPipeEventPrvGCMarkHandles_V1 = nullptr;
EventPipeEvent *EventPipeEventPrvGCMarkCards_V1 = nullptr;
EventPipeEvent *EventPipeEventBGCBegin = nullptr;
EventPipeEvent *EventPipeEventBGC1stNonConEnd = nullptr;
EventPipeEvent *EventPipeEventBGC1stConEnd = nullptr;
EventPipeEvent *EventPipeEventBGC2ndNonConBegin = nullptr;
EventPipeEvent *EventPipeEventBGC2ndNonConEnd = nullptr;
EventPipeEvent *EventPipeEventBGC2ndConBegin = nullptr;
EventPipeEvent *EventPipeEventBGC2ndConEnd = nullptr;
EventPipeEvent *EventPipeEventBGCPlanEnd = nullptr;
EventPipeEvent *EventPipeEventBGCSweepEnd = nullptr;
EventPipeEvent *EventPipeEventBGCDrainMark = nullptr;
EventPipeEvent *EventPipeEventBGCRevisit = nullptr;
EventPipeEvent *EventPipeEventBGCOverflow_V1 = nullptr;
EventPipeEvent *EventPipeEventBGCAllocWaitBegin = nullptr;
EventPipeEvent *EventPipeEventBGCAllocWaitEnd = nullptr;
EventPipeEvent *EventPipeEventGCFullNotify_V1 = nullptr;
EventPipeEvent *EventPipeEventBGC1stSweepEnd = nullptr;
EventPipeEvent *EventPipeEventFailFast = nullptr;
EventPipeEvent *EventPipeEventPrvFinalizeObject = nullptr;
EventPipeEvent *EventPipeEventPrvSetGCHandle = nullptr;
EventPipeEvent *EventPipeEventPrvDestroyGCHandle = nullptr;
EventPipeEvent *EventPipeEventPinPlugAtGCTime = nullptr;

BOOL EventPipeEventEnabledGCDecision_V1(void)
{
    return ep_event_is_enabled(EventPipeEventGCDecision_V1);
}

ULONG EventPipeWriteEventGCDecision_V1(
    const BOOL DoCompact,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledGCDecision_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(DoCompact, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventGCDecision_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledGCSettings_V1(void)
{
    return ep_event_is_enabled(EventPipeEventGCSettings_V1);
}

ULONG EventPipeWriteEventGCSettings_V1(
    const uint64_t SegmentSize,
    const uint64_t LargeObjectSegmentSize,
    const BOOL ServerGC,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledGCSettings_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(SegmentSize, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(LargeObjectSegmentSize, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ServerGC, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventGCSettings_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledGCOptimized_V1(void)
{
    return ep_event_is_enabled(EventPipeEventGCOptimized_V1);
}

ULONG EventPipeWriteEventGCOptimized_V1(
    const uint64_t DesiredAllocation,
    const uint64_t NewAllocation,
    const unsigned int GenerationNumber,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledGCOptimized_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(DesiredAllocation, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(NewAllocation, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(GenerationNumber, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventGCOptimized_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledGCGlobalHeapHistory(void)
{
    return ep_event_is_enabled(EventPipeEventGCGlobalHeapHistory);
}

ULONG EventPipeWriteEventGCGlobalHeapHistory(
    const uint64_t FinalYoungestDesired,
    const signed int NumHeaps,
    const unsigned int CondemnedGeneration,
    const unsigned int Gen0ReductionCount,
    const unsigned int Reason,
    const unsigned int GlobalMechanisms,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledGCGlobalHeapHistory())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(FinalYoungestDesired, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(NumHeaps, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(CondemnedGeneration, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Gen0ReductionCount, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Reason, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(GlobalMechanisms, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventGCGlobalHeapHistory, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvGCMarkStackRoots_V1(void)
{
    return ep_event_is_enabled(EventPipeEventPrvGCMarkStackRoots_V1);
}

ULONG EventPipeWriteEventPrvGCMarkStackRoots_V1(
    const unsigned int HeapNum,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvGCMarkStackRoots_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HeapNum, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvGCMarkStackRoots_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvGCMarkFinalizeQueueRoots_V1(void)
{
    return ep_event_is_enabled(EventPipeEventPrvGCMarkFinalizeQueueRoots_V1);
}

ULONG EventPipeWriteEventPrvGCMarkFinalizeQueueRoots_V1(
    const unsigned int HeapNum,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvGCMarkFinalizeQueueRoots_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HeapNum, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvGCMarkFinalizeQueueRoots_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvGCMarkHandles_V1(void)
{
    return ep_event_is_enabled(EventPipeEventPrvGCMarkHandles_V1);
}

ULONG EventPipeWriteEventPrvGCMarkHandles_V1(
    const unsigned int HeapNum,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvGCMarkHandles_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HeapNum, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvGCMarkHandles_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvGCMarkCards_V1(void)
{
    return ep_event_is_enabled(EventPipeEventPrvGCMarkCards_V1);
}

ULONG EventPipeWriteEventPrvGCMarkCards_V1(
    const unsigned int HeapNum,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvGCMarkCards_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HeapNum, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvGCMarkCards_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCBegin(void)
{
    return ep_event_is_enabled(EventPipeEventBGCBegin);
}

ULONG EventPipeWriteEventBGCBegin(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCBegin())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCBegin, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC1stNonConEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGC1stNonConEnd);
}

ULONG EventPipeWriteEventBGC1stNonConEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC1stNonConEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC1stNonConEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC1stConEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGC1stConEnd);
}

ULONG EventPipeWriteEventBGC1stConEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC1stConEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC1stConEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC2ndNonConBegin(void)
{
    return ep_event_is_enabled(EventPipeEventBGC2ndNonConBegin);
}

ULONG EventPipeWriteEventBGC2ndNonConBegin(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC2ndNonConBegin())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC2ndNonConBegin, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC2ndNonConEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGC2ndNonConEnd);
}

ULONG EventPipeWriteEventBGC2ndNonConEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC2ndNonConEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC2ndNonConEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC2ndConBegin(void)
{
    return ep_event_is_enabled(EventPipeEventBGC2ndConBegin);
}

ULONG EventPipeWriteEventBGC2ndConBegin(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC2ndConBegin())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC2ndConBegin, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC2ndConEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGC2ndConEnd);
}

ULONG EventPipeWriteEventBGC2ndConEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC2ndConEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC2ndConEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCPlanEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGCPlanEnd);
}

ULONG EventPipeWriteEventBGCPlanEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCPlanEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCPlanEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCSweepEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGCSweepEnd);
}

ULONG EventPipeWriteEventBGCSweepEnd(
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCSweepEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCSweepEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCDrainMark(void)
{
    return ep_event_is_enabled(EventPipeEventBGCDrainMark);
}

ULONG EventPipeWriteEventBGCDrainMark(
    const uint64_t Objects,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCDrainMark())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(Objects, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCDrainMark, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCRevisit(void)
{
    return ep_event_is_enabled(EventPipeEventBGCRevisit);
}

ULONG EventPipeWriteEventBGCRevisit(
    const uint64_t Pages,
    const uint64_t Objects,
    const unsigned int IsLarge,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCRevisit())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(Pages, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Objects, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(IsLarge, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCRevisit, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCOverflow_V1(void)
{
    return ep_event_is_enabled(EventPipeEventBGCOverflow_V1);
}

ULONG EventPipeWriteEventBGCOverflow_V1(
    const uint64_t Min,
    const uint64_t Max,
    const uint64_t Objects,
    const unsigned int IsLarge,
    const unsigned short ClrInstanceID,
    const unsigned int GenNumber,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCOverflow_V1())
        return ERROR_SUCCESS;

    size_t size = 34;
    BYTE stackBuffer[34];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(Min, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Max, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Objects, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(IsLarge, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(GenNumber, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCOverflow_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCAllocWaitBegin(void)
{
    return ep_event_is_enabled(EventPipeEventBGCAllocWaitBegin);
}

ULONG EventPipeWriteEventBGCAllocWaitBegin(
    const unsigned int Reason,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCAllocWaitBegin())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(Reason, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCAllocWaitBegin, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGCAllocWaitEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGCAllocWaitEnd);
}

ULONG EventPipeWriteEventBGCAllocWaitEnd(
    const unsigned int Reason,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGCAllocWaitEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(Reason, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGCAllocWaitEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledGCFullNotify_V1(void)
{
    return ep_event_is_enabled(EventPipeEventGCFullNotify_V1);
}

ULONG EventPipeWriteEventGCFullNotify_V1(
    const unsigned int GenNumber,
    const unsigned int IsAlloc,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledGCFullNotify_V1())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(GenNumber, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(IsAlloc, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventGCFullNotify_V1, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledBGC1stSweepEnd(void)
{
    return ep_event_is_enabled(EventPipeEventBGC1stSweepEnd);
}

ULONG EventPipeWriteEventBGC1stSweepEnd(
    const unsigned int GenNumber,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledBGC1stSweepEnd())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(GenNumber, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventBGC1stSweepEnd, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledFailFast(void)
{
    return ep_event_is_enabled(EventPipeEventFailFast);
}

ULONG EventPipeWriteEventFailFast(
    const WCHAR* FailFastUserMessage,
    const void* FailedEIP,
    const unsigned int OSExitCode,
    const unsigned int ClrExitCode,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledFailFast())
        return ERROR_SUCCESS;

    size_t size = 82;
    BYTE stackBuffer[82];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    if (!FailFastUserMessage) { FailFastUserMessage = W("NULL"); }
    success &= WriteToBuffer(FailFastUserMessage, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(FailedEIP, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(OSExitCode, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrExitCode, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventFailFast, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvFinalizeObject(void)
{
    return ep_event_is_enabled(EventPipeEventPrvFinalizeObject);
}

ULONG EventPipeWriteEventPrvFinalizeObject(
    const void* TypeID,
    const void* ObjectID,
    const unsigned short ClrInstanceID,
    const WCHAR* TypeName,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvFinalizeObject())
        return ERROR_SUCCESS;

    size_t size = 82;
    BYTE stackBuffer[82];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    if (!TypeName) { TypeName = W("NULL"); }
    success &= WriteToBuffer(TypeID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ObjectID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(TypeName, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvFinalizeObject, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvSetGCHandle(void)
{
    return ep_event_is_enabled(EventPipeEventPrvSetGCHandle);
}

ULONG EventPipeWriteEventPrvSetGCHandle(
    const void* HandleID,
    const void* ObjectID,
    const unsigned int Kind,
    const unsigned int Generation,
    const uint64_t AppDomainID,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvSetGCHandle())
        return ERROR_SUCCESS;

    size_t size = 34;
    BYTE stackBuffer[34];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HandleID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ObjectID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Kind, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(Generation, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(AppDomainID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvSetGCHandle, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPrvDestroyGCHandle(void)
{
    return ep_event_is_enabled(EventPipeEventPrvDestroyGCHandle);
}

ULONG EventPipeWriteEventPrvDestroyGCHandle(
    const void* HandleID,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPrvDestroyGCHandle())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(HandleID, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPrvDestroyGCHandle, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

BOOL EventPipeEventEnabledPinPlugAtGCTime(void)
{
    return ep_event_is_enabled(EventPipeEventPinPlugAtGCTime);
}

ULONG EventPipeWriteEventPinPlugAtGCTime(
    const void* PlugStart,
    const void* PlugEnd,
    const void* GapBeforeSize,
    const unsigned short ClrInstanceID,
    const GUID * ActivityId,
    const GUID * RelatedActivityId)
{
    if (!EventPipeEventEnabledPinPlugAtGCTime())
        return ERROR_SUCCESS;

    size_t size = 32;
    BYTE stackBuffer[32];
    BYTE *buffer = stackBuffer;
    size_t offset = 0;
    bool fixedBuffer = true;
    bool success = true;

    success &= WriteToBuffer(PlugStart, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(PlugEnd, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(GapBeforeSize, buffer, offset, size, fixedBuffer);
    success &= WriteToBuffer(ClrInstanceID, buffer, offset, size, fixedBuffer);

    if (!success)
    {
        if (!fixedBuffer)
            delete[] buffer;
        return ERROR_WRITE_FAULT;
    }

    ep_write_event (EventPipeEventPinPlugAtGCTime, (uint8_t *)buffer, (uint32_t)offset, reinterpret_cast<const uint8_t*>(ActivityId), reinterpret_cast<const uint8_t*>(RelatedActivityId));

    if (!fixedBuffer)
        delete[] buffer;


    return ERROR_SUCCESS;
}

void InitDotNETRuntimePrivate(void);

void InitDotNETRuntimePrivate(void)
{
    EventPipeProviderDotNETRuntimePrivate = create_provider(DotNETRuntimePrivateName, reinterpret_cast<EventPipeCallback>(EventPipeEtwCallbackDotNETRuntimePrivate));
    EventPipeEventGCDecision_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,1,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventGCSettings_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,2,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventGCOptimized_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,3,1,1,EP_EVENT_LEVEL_VERBOSE,true,NULL,0);
    EventPipeEventGCGlobalHeapHistory = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,5,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventPrvGCMarkStackRoots_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,7,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventPrvGCMarkFinalizeQueueRoots_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,8,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventPrvGCMarkHandles_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,9,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventPrvGCMarkCards_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,10,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCBegin = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,11,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC1stNonConEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,12,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC1stConEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,13,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC2ndNonConBegin = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,14,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC2ndNonConEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,15,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC2ndConBegin = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,16,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC2ndConEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,17,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCPlanEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,18,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCSweepEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,19,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCDrainMark = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,20,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCRevisit = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,21,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCOverflow_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,22,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCAllocWaitBegin = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,23,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGCAllocWaitEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,24,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventGCFullNotify_V1 = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,25,1,1,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventBGC1stSweepEnd = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,26,1,0,EP_EVENT_LEVEL_INFORMATIONAL,true,NULL,0);
    EventPipeEventFailFast = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,191,0,0,EP_EVENT_LEVEL_CRITICAL,true,NULL,0);
    EventPipeEventPrvFinalizeObject = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,192,1,0,EP_EVENT_LEVEL_VERBOSE,true,NULL,0);
    EventPipeEventPrvSetGCHandle = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,194,16384,0,EP_EVENT_LEVEL_VERBOSE,true,NULL,0);
    EventPipeEventPrvDestroyGCHandle = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,195,16384,0,EP_EVENT_LEVEL_VERBOSE,true,NULL,0);
    EventPipeEventPinPlugAtGCTime = ep_provider_add_event(EventPipeProviderDotNETRuntimePrivate,199,1,0,EP_EVENT_LEVEL_VERBOSE,true,NULL,0);
}
